@@ -1,5 +1,3 @@
-import re
-
 from asyncpg import connection
 from sqlalchemy.dialects.postgresql import pypostgresql
 from sqlalchemy.sql import ClauseElement
@@ -7,50 +5,13 @@ from sqlalchemy.sql.dml import Insert as InsertObject, Update as UpdateObject
 
 from .log import query_logger
 
-_dialect = pypostgresql.dialect()
+_dialect = pypostgresql.dialect(paramstyle='pyformat')
 _dialect.implicit_returning = True
 _dialect.supports_native_enum = True
 _dialect.supports_smallserial = True  # 9.2+
 _dialect._backslash_escapes = False
 _dialect.supports_sane_multi_rowcount = True  # psycopg 2.0.9+
 _dialect._has_native_hstore = True
-_dialect.paramstyle = 'named'
-_pattern = r'(?<!:):(\w+)'
-_compiled_pattern = re.compile(_pattern, re.M)
-
-
-class MissingParameterError(KeyError):
-    """ This error gets thrown when a parameter is missing in a query """
-
-
-def _replace_keys(querystring, params, inline=False):
-    new_params = []
-    for index, param in enumerate(params):
-        name, value = param
-        if inline:
-            querystring = querystring.replace(':' + name, value, 1)
-        else:
-            querystring = querystring.replace(':' + name,
-                                              '$' + str(index + 1), 1)
-            new_params.append(value)
-
-    return querystring, new_params
-
-
-def _get_keys(compiled):
-    p = _compiled_pattern
-    keys = re.findall(p, compiled.string)
-    processors = compiled._bind_processors
-    params = []
-    try:
-        for key in keys:
-            processed_param = (processors[key](compiled.params[key])
-                               if key in processors
-                               else compiled.params[key])
-            params.append((key, processed_param))
-    except KeyError as e:
-        raise MissingParameterError('Parameter {} missing'.format(e))
-    return params
 
 
 def execute_defaults(query):
@@ -61,16 +22,24 @@ def execute_defaults(query):
     else:
         return query
 
-    query.parameters = query.parameters or {}
+    # query.parameters could be a list in a multi row insert
+    if isinstance(query.parameters, list):
+        for param in query.parameters:
+            _execute_default_attr(query, param, attr_name)
+    else:
+        query.parameters = query.parameters or {}
+        _execute_default_attr(query, query.parameters, attr_name)
+    return query
+
+
+def _execute_default_attr(query, param, attr_name):
     for col in query.table.columns:
         attr = getattr(col, attr_name)
-        if attr and query.parameters.get(col.name) is None:
+        if attr and param.get(col.name) is None:
             if attr.is_scalar:
-                query.parameters[col.name] = attr.arg
-            elif col.default and col.default.is_callable:
-                query.parameters[col.name] = attr.arg({})
-
-    return query
+                param[col.name] = attr.arg
+            elif attr.is_callable:
+                param[col.name] = attr.arg({})
 
 
 def compile_query(query, dialect=_dialect, inline=False):
@@ -80,8 +49,15 @@ def compile_query(query, dialect=_dialect, inline=False):
     elif isinstance(query, ClauseElement):
         query = execute_defaults(query)  # default values for Insert/Update
         compiled = query.compile(dialect=dialect)
-        params = _get_keys(compiled)
-        new_query, new_params = _replace_keys(compiled.string, params)
+        compiled_params = sorted(compiled.params.items())
+
+        mapping = {key: '$' + str(i)
+                   for i, (key, _) in enumerate(compiled_params, start=1)}
+        new_query = compiled.string % mapping
+
+        processors = compiled._bind_processors
+        new_params = [processors[key](val) if key in processors else val
+                      for key, val in compiled_params]
 
         query_logger.debug(new_query)
 
